﻿local _G = getfenv(0);

-- Constants
local modName = "QBar";
local btnSize = 36;
local slots = {
  "HeadSlot", "NeckSlot", "ShoulderSlot", "BackSlot", "ChestSlot", "ShirtSlot", "TabardSlot", "WristSlot",
  "HandsSlot", "WaistSlot", "LegsSlot", "FeetSlot", "Finger0Slot", "Finger1Slot", "Trinket0Slot", "Trinket1Slot",
  "MainHandSlot", "SecondaryHandSlot", "RangedSlot",
};

-- Work Vars
local shownItems = 0;
local lastName;
local bindFrame;
local updateTime = 0;
local x, y;

-- Config
QBar_Config = {};
local cfg;
local defConfig = {
  enabled = true,
  showTips = true,
  vertical = false,
  downward = true,
  leftside = true,
  scale = 0.62,
  padding = 7,
};

QBar_userDefindedList={};

--------------------------------------------------------------------------------------------------------
--                                                Main                                                --
--------------------------------------------------------------------------------------------------------

local f = CreateFrame("Frame",modName,UIParent);
f:SetMovable(1);
f:SetToplevel(1);
f:SetFrameStrata("LOW");
f:SetClampedToScreen(true);
f:SetScript("OnDragStart",function(self)
  if IsShiftKeyDown() then
    self:StartMoving();
  end
end);
f:SetScript("OnDragStop",function(self)
  self:StopMovingOrSizing();
  if cfg.leftside then
    cfg.left, cfg.bottom = self:GetLeft(), self:GetBottom();
  else
    cfg.left, cfg.bottom = self:GetRight(), self:GetBottom();
  end
end);
f:RegisterForDrag("LeftButton");
f:SetHitRectInsets(-10,0,0,0);

f:SetScript("OnEnter",function(self)
  GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT");
  GameTooltip:SetText("이곳을 쉬프트+클릭 드래그하면 바를 이동할 수 있습니다.");
  GameTooltip:Show();
end);
f:SetScript("OnLeave",function(self) GameTooltip:Hide() end);

local function OnUpdate(self,elapsed)
  updateTime = (updateTime + elapsed);
  if (updateTime > 0.25) then
    f.UpdateButtons();
    f:SetScript("OnUpdate",nil);
  end
end

f.tip = CreateFrame("GameTooltip",modName.."Tip",nil,"GameTooltipTemplate");
f.tip:SetOwner(UIParent,"ANCHOR_NONE");

--------------------------------------------------------------------------------------------------------
--                                             Local Funcs                                            --
--------------------------------------------------------------------------------------------------------

-- Lock Frame
local function SetLocked(lock)
  if (lock) then
    f:EnableMouse(false);
    --f:SetBackdrop(nil);
  else
    f:EnableMouse(true);
    --f:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = 1, tileSize = 16, insets = { left = 0, right = 0, top = 0, bottom = 0 } });
    --f:SetBackdropColor(0.1,0.22,0.35,1);
  end
end

-- Bindings Frame
local function OpenBindingFrame()
  if (bindFrame) then
    bindFrame:Show();
    return;
  end

  bindFrame = CreateFrame("Frame",nil,UIParent);
  bindFrame:SetWidth(200);
  bindFrame:SetHeight(60);
  bindFrame:SetPoint("CENTER");
  bindFrame:SetFrameStrata("DIALOG");
  bindFrame:EnableKeyboard(1);
  bindFrame:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 16, edgeSize = 16, insets = { left = 4, right = 4, top = 4, bottom = 4 } });
  bindFrame:SetBackdropColor(0.1,0.22,0.35,1);
  bindFrame:SetBackdropBorderColor(0.1,0.1,0.1,1);

  function bindFrame.keyDownFunc(self,key)
    -- Check Key
    if (key == "ESCAPE") then
      if (cfg.bindKey) then
        SetBinding(cfg.bindKey);
        cfg.bindKey = nil;
        f.UpdateBinding();
      end;
      self:Hide();
      return;
    elseif (key:match("^.SHIFT") or key:match("^.CTRL") or key:match("^.ALT")) then
      return;
    end
    -- Prefix Modifier Key
    if (IsShiftKeyDown()) then
      key = "SHIFT-"..key;
    end
    if (IsControlKeyDown()) then
      key = "CTRL-"..key;
    end
    if (IsAltKeyDown()) then
      key = "ALT-"..key;
    end
    -- Validate
    if (GetBindingAction(key) ~= "") then
      QBar_Print("|_2오류|r: 단축키 |_1"..key.."|r 은(는) 이미 |_1"..GetBindingAction(key).."|r에 지정되어 있습니다.");
      return;
    end
    if (cfg.bindKey) then
      SetBinding(cfg.bindKey);
    end
    -- Set New Hotkey
    cfg.bindKey = key;
    QBar_Print("|_2"..modName.."|r: 단축키가 |_1"..key.."|r 로 지정되었습니다.");
    f.UpdateBinding(1);
    self:Hide();
  end
  bindFrame:SetScript("OnKeyDown",bindFrame.keyDownFunc);

  bindFrame.hint = bindFrame:CreateFontString(nil,"ARTWORK","GameFontNormal");
  bindFrame.hint:SetPoint("CENTER");
  bindFrame.hint:SetText("지정하려는 키를 누르세요...\nEscape : 키지정 해제");
end

-- SetEnabled
local function SetEnabledStatus()
  if (cfg.enabled) then
    f:RegisterEvent("BAG_UPDATE");
    f:RegisterEvent("UNIT_INVENTORY_CHANGED");
    f:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
    f:RegisterEvent("PLAYER_REGEN_ENABLED");
    f:RegisterEvent("UPDATE_BINDINGS");
    f:Show();
    f.RequestUpdate();
  else
    f:UnregisterAllEvents();
    f:Hide();
  end
end

--------------------------------------------------------------------------------------------------------
--                                               Events                                               --
--------------------------------------------------------------------------------------------------------
local function OnEvent(self,event,p1,...)
  -- Variables Loaded
  if (event == "VARIABLES_LOADED") then
    cfg = QBar_Config;
    f:ClearAllPoints();
    if (cfg.left and cfg.bottom) then
      if cfg.leftside then
        f:SetPoint("BOTTOMLEFT","UIParent","BOTTOMLEFT",cfg.left,cfg.bottom);
      else
        f:SetPoint("BOTTOMRIGHT","UIParent","BOTTOMLEFT",cfg.left,cfg.bottom);
      end
    else
      f:SetPoint("CENTER");
    end
    for option, value in pairs(defConfig) do
      if (cfg[option] == nil) then
        cfg[option] = value;
      end
    end
    SetEnabledStatus();
    f:SetScale(cfg.scale);
  -- Update Cooldowns
  elseif (event == "ACTIONBAR_UPDATE_COOLDOWN") then
    if (shownItems > 0) then
      f.UpdateCooldowns();
    end
  -- Bindings Update
  elseif (event == "UPDATE_BINDINGS") then
    f.UpdateBinding();
  -- Inventory Changed
  elseif (event == "UNIT_INVENTORY_CHANGED") then
    if (p1 == "player") then
      f.RequestUpdate();
    end
  -- Other Events
  else
    f.RequestUpdate();
  end
end

f:SetScript("OnEvent",OnEvent);
f:RegisterEvent("VARIABLES_LOADED");
--------------------------------------------------------------------------------------------------------
--                                            Command Line                                            --
--------------------------------------------------------------------------------------------------------
_G["SLASH_"..modName.."1"] = "/qb";
SlashCmdList[modName] = function(cmd)
  -- Extract Parameters
  local param1, param2 = cmd:match("^([^%s]+)%s*(.*)$");
  param1 = (param1 and param1:lower() or cmd:lower());
  -- Enabled
  if (param1 == "toggle") then
    cfg.enabled = not cfg.enabled;
    SetEnabledStatus();
    QBar_Print("|_2"..modName.."|r 기능이 |_1"..(cfg.enabled and "활성화 되었습니다." or "중지 되었습니다.").."|r.");
  -- Scale
  elseif (param1 == "scale") then
    if (param2 ~= "") and (type(tonumber(param2)) == "number") then
      cfg.scale = tonumber(param2);
      f:SetScale(cfg.scale);
      QBar_Print("버튼 스케일을 |_1"..cfg.scale.."|r|1으로;로; 설정합니다.");
    end
  -- Padding
  elseif (param1 == "padding") then
    if (param2 ~= "") and (type(tonumber(param2)) == "number") then
      cfg.padding = tonumber(param2);
      f.RequestUpdate();
      QBar_Print("버튼간의 간격을 |_1"..cfg.padding.."|r|1으로;로; 설정합니다.");
    end
  -- Tips
  elseif (param1 == "tips") then
    cfg.showTips = not cfg.showTips;
    QBar_Print("아이템의 툴팁을 표시|_1"..(cfg.showTips and "합니다." or "하지 않습니다.").."|r.");
  -- Vertical
  elseif (param1 == "vertical") then
    cfg.vertical = not cfg.vertical;
    f.RequestUpdate();
    QBar_Print("수평 정렬|_1"..(cfg.vertical and "을 사용합니다." or "을 사용하지 않습니다.").."|r.");
  elseif (param1 == "downward") then
    cfg.downward = not cfg.downward;
    f:ClearAllPoints();
    if cfg.downward then
      f:SetPoint("TOPLEFT","UIParent","BOTTOMLEFT",cfg.left,cfg.bottom);
    else
      f:SetPoint("BOTTOMLEFT","UIParent","BOTTOMLEFT",cfg.left,cfg.bottom);
    end
    f.RequestUpdate();
    QBar_Print("수직 정렬시|_1"..(cfg.downward and " 아래쪽으로 정렬 합니다." or " 위쪽으로 정렬합니다.").."|r.");
  elseif (param1 == "leftside") then  
    cfg.leftside = not cfg.leftside;
    f:ClearAllPoints();
    if cfg.leftside then
      f:SetPoint("BOTTOMLEFT","UIParent","BOTTOMLEFT",cfg.left,cfg.bottom);
    else
      f:SetPoint("BOTTOMRIGHT","UIParent","BOTTOMLEFT",cfg.left,cfg.bottom);
    end
    f.RequestUpdate();
    QBar_Print("수평 정렬시|_1"..(cfg.leftside and " 왼쪽으로 정렬합니다." or " 오른쪽으로 정렬합니다.").."|r.");
  -- Lock
  elseif (param1 == "lock") then
    SetLocked(f:IsMouseEnabled());
    QBar_Print("QBar 버튼 프레임|_1"..(f:IsMouseEnabled() and "을 이동 할 수 있습니다." or " 위치가 고정되었습니다.").."|r.");
  -- Bind
  elseif (param1 == "bind") then
    OpenBindingFrame();
  -- Clear Ignore
  elseif (param1 == "clearignore") then
    QBar_ignoreList = nil;
    f.RequestUpdate();
    QBar_Print("제외 목록이 초기화 되었습니다.");
  -- Add Item in User Definded List
  elseif (param1 == "add") then
    if (param2 ~= "") and GetItemInfo(param2) then
      local tmpItemName = GetItemInfo(param2);
      QBar_userDefindedList[tmpItemName]=1;
      QBar_Print("|_1"..tmpItemName.."|r|1이;가; 사용자 목록에 |_2추가|r 되었습니다.");
      f.RequestUpdate();
    else
      QBar_Print("아이템 추가 오류: 해당 아이템을 쉬프트+클릭으로 링크하십시요.");
    end
  -- Invalid or No Command
  else
    UpdateAddOnMemoryUsage();
    QBar_Print(format("----- |_2%s|r |_1%s|r ----- |_1%.2f |_2kb|r -----",modName,GetAddOnMetadata(modName,"Version"),GetAddOnMemoryUsage(modName)));
    QBar_Print("The following |_2parameters|r are valid for this addon:");
    QBar_Print(" |_2toggle|r = QBAR 사용/중지 토글.");
    QBar_Print(" |_2scale 'value'|r = 버튼 크기(스케일)를 조절합니다.");
    QBar_Print(" |_2padding 'value'|r = 버튼 간격을 조절합니다.");
    QBar_Print(" |_2tips|r = 툴팁 표시 여부를 토글합니다");
    QBar_Print(" |_2vertical|r = 버튼 정렬 방법 설정(수평/수직)");
    QBar_Print(" |_2downward|r = 버튼 수직 정렬시 방향 설정(아래/위)");
    QBar_Print(" |_2leftside|r = 버튼 수평 정렬시 방향 설정(왼쪽/오른쪽)");
    QBar_Print(" |_2lock|r = 버튼 이동 잠금/해제.");
    QBar_Print(" |_2clearignore|r = 제외 목록을 초기화 하고 모든 아이템을 보여줍니다.");
    QBar_Print(" |_2add [itemlink]|r = 해당 아이템을 사용자 정의 목록에 추가합니다.");
    QBar_Print(" |_2bind|r = 가장 최근에 사용한 아이템에 대한 단축키를 지정합니다.");
  end
end
--------------------------------------------------------------------------------------------------------
--                                                Items                                               --
--------------------------------------------------------------------------------------------------------

f.items = {};

-- Button Scripts
local function Button_OnEnter(self)
  if (cfg.showTips) then
    GameTooltip:SetOwner(self,"ANCHOR_RIGHT");
    local bag, slot = self:GetAttribute("bag"), self:GetAttribute("slot");
    if (bag) then
      GameTooltip:SetBagItem(bag,slot);
    else
      GameTooltip:SetInventoryItem("player",slot);
    end
  end
end

local function HideGTT()
  GameTooltip:Hide();
end

local function Button_OnClick(self,button)
  -- Handle Modified Click
  if (HandleModifiedItemClick(self.link)) then
    return;
  -- Ignore
  elseif (IsShiftKeyDown()) then
    if QBar_userDefindedList[self.name] then
      QBar_userDefindedList[self.name]=nil;
      QBar_Print("|_1"..self.name.."|r|1이;가; 사용자 목록에서 |_2제거|r 되었습니다.");
      f.RequestUpdate();
      return;
    end
    if (not QBar_ignoreList) then
      QBar_ignoreList = {};
    end
    QBar_ignoreList[self.name] = true;
    f.RequestUpdate();
  -- Set Hotkey
  else
    lastName = self.name;
    f.UpdateBinding(1);
  end
end

-- Make Loot Button
function QBar_MakeItemButton()
  local b = CreateFrame("Button",modName..(#f.items + 1),f,"SecureActionButtonTemplate");
  b:SetWidth(btnSize);
  b:SetHeight(btnSize);
  b:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square");
  b:RegisterForClicks("LeftButtonUp","RightButtonUp");
  b:SetScript("OnEnter",Button_OnEnter);
  b:SetScript("OnLeave",HideGTT);
  b:HookScript("OnClick",Button_OnClick);
  b:SetAttribute("type*","item");

  b.icon = b:CreateTexture("$parentIcon","ARTWORK");
  b.icon:SetAllPoints();

  b.count = b:CreateFontString(nil,"ARTWORK","NumberFontNormal");
  b.count:SetPoint("BOTTOMRIGHT",b.icon,-3,3);

  b.cooldown = CreateFrame("Cooldown",nil,b,"CooldownFrameTemplate");
  b.cooldown:SetAllPoints();

  b.bind = b:CreateFontString(nil,"ARTWORK","NumberFontNormalSmallGray");
  b.bind:SetPoint("TOPLEFT",b.icon,3,-3);
  b.bind:SetPoint("TOPRIGHT",b.icon,-3,-3);
  b.bind:SetJustifyH("RIGHT");

  if (#f.items == 0) then
    if cfg.leftside then
      b:SetPoint("TOPLEFT",f,"TOPLEFT",8,-8);
    else
      b:SetPoint("TOPRIGHT",f,"TOPRIGHT",-8,-8);
    end
  end

  tinsert(f.items,b);
  return b;
end

-- Add Button
local function AddButton(index,bag,slot,link,count)
  button = f.items[index] or QBar_MakeItemButton();

  button.icon:SetTexture(select(10,GetItemInfo(link)));
  button.count:SetText(count and count > 1 and count or "");

  button.link = link;
  button.name = GetItemInfo(link);

  button:SetAttribute("bag",bag);
  button:SetAttribute("slot",slot);

  if (index == 1) then
    button:ClearAllPoints();
    if (cfg.vertical) then
      if cfg.downward then
        button:SetPoint("TOP",f,"TOP",0,-cfg.padding);
      else
        button:SetPoint("BOTTOM",f,"BOTTOM",0,cfg.padding);
      end 
    else
      if cfg.leftside then
        button:SetPoint("LEFT",f,"LEFT",cfg.padding,0);
      else 
        button:SetPoint("RIGHT",f,"RIGHT",-cfg.padding,0);
      end
    end
  else
    button:ClearAllPoints();
    if (cfg.vertical) then
      if cfg.downward then
        button:SetPoint("TOP",f.items[index - 1],"BOTTOM",0,-cfg.padding);
      else
        button:SetPoint("BOTTOM",f.items[index - 1],"TOP",0,cfg.padding);
      end 
    else
      if cfg.leftside then
        button:SetPoint("LEFT",f.items[index - 1],"RIGHT",cfg.padding,0);
      else 
        button:SetPoint("RIGHT",f.items[index - 1],"LEFT",-cfg.padding,0);
      end
    end
  end

  button:Show();
end

-- Check Item
local function CheckItem(link)
  local itemName, _, _, _, _, itemType, itemSubType = GetItemInfo(link);
  if (QBar_ignoreList) and (QBar_ignoreList[itemName]) then
    QBar_Print("|_1"..itemName.."|r|1은;는; 사용자 제외목록에 들어있는 아이템입니다.");
    return;
  end
  f.tip:ClearLines();
  f.tip:SetHyperlink(link);
  local text;
  if (f.tip:NumLines() >= 3) and ((itemType == "Quest" or itemSubType == "Quest" or
      _G[modName.."TipTextLeft2"]:GetText() == ITEM_BIND_QUEST) or QBar_userDefindedList[itemName]) then
    for i = 3, f.tip:NumLines() do
      text = _G[modName.."TipTextLeft"..i]:GetText() or "";
--      if (text:find("^"..ITEM_SPELL_TRIGGER_ONUSE) or text:find("^"..ITEM_STARTS_QUEST)) then
      if (text:find("^"..ITEM_SPELL_TRIGGER_ONUSE)) or (text:find("^"..ITEM_SPELL_TRIGGER_ONEQUIP)) then
        return 1;
      end
    end
  end
  return;
end

--------------------------------------------------------------------------------------------------------
--                                               Update                                               --
--------------------------------------------------------------------------------------------------------

-- Request a Button Update
function f.RequestUpdate()
  updateTime = 0;
  f:SetScript("OnUpdate",OnUpdate);
end

-- Update Buttons
function f.UpdateButtons()
  -- Check if we are locked by combat
  if (InCombatLockdown()) then
    return;
  end
  -- locals
  local index = 1;
  local link, slotId;
  -- Inventory
  for bag = 0, NUM_BAG_FRAMES do
    for slot = 1, GetContainerNumSlots(bag) do
      link = GetContainerItemLink(bag,slot);
      if (link) and (CheckItem(link)) then
        AddButton(index,bag,slot,link,select(2,GetContainerItemInfo(bag,slot)));
        index = (index + 1);
      end
    end
  end
  -- Equipped Items
  for _, slotName in ipairs(slots) do
    slotId = GetInventorySlotInfo(slotName);
    link = GetInventoryItemLink("player",slotId);
    if (link) and (CheckItem(link)) then
      AddButton(index,nil,slotId,link);
      index = (index + 1);
    end
  end
  -- Set Shown Items
  shownItems = (index - 1);
  for i = index, #f.items do
    f.items[i]:Hide();
  end
  -- Set Frame Dimention
  x = (btnSize * shownItems + (shownItems - 1) * cfg.padding + 16);
  y = (btnSize + 16);
  f:SetWidth(cfg.vertical and y or x);
  f:SetHeight(cfg.vertical and x or y);
  -- Update Misc
  f.UpdateBinding(1);
  f.UpdateCooldowns();
end

-- Update Cooldowns
function f.UpdateCooldowns()
  local bag, slot;
  for i = 1, shownItems do
    bag, slot = f.items[i]:GetAttribute("bag"), f.items[i]:GetAttribute("slot");
    if (bag) then
      CooldownFrame_SetTimer(f.items[i].cooldown,GetContainerItemCooldown(bag,slot));
    else
      CooldownFrame_SetTimer(f.items[i].cooldown,GetInventoryItemCooldown("player",slot));
    end
  end
end

-- Update Bind Text
function f.UpdateBinding(reBind)
  -- Can't rebind in combat
  if (InCombatLockdown()) and (reBind) then
    reBind = nil;
  end
  -- Unbind previous button first
  if (reBind) and (cfg.bindKey) then
    SetBinding(cfg.bindKey);
  end
  -- Update all visible buttons
  for i = 1, shownItems do
    if (reBind) and (cfg.bindKey) and (lastName == f.items[i].name) then
      SetBindingClick(cfg.bindKey,modName..i);
    end
    f.items[i].bind:SetText(GetBindingText(GetBindingKey("CLICK "..modName..i..":LeftButton"),"",1));
  end
end

--------------------------------------------------------------------------------------------------------
--                                Global Chat Message Function                                --
--------------------------------------------------------------------------------------------------------
function QBar_Print(text)
  DEFAULT_CHAT_FRAME:AddMessage("QBar->"..tostring(text):gsub("|_1","|cffffff80"):gsub("|_2","|cffffffff"),128/255,192/255,255/255);
end

function QBar_ContainerFrameItemButton_OnModifiedClick(self,button)
  if (button == "RightButton") and IsControlKeyDown() then
    local itemLink = GetContainerItemLink(self:GetParent():GetID(), self:GetID());
    local itemName = itemLink and GetItemInfo(itemLink) or nil;
    if ( itemName ) then
      QBar_userDefindedList[itemName]=1;
      if CheckItem(itemLink) then
        QBar_Print("|_1"..itemName.."|r|1이;가; 사용자 목록에 |_2추가|r 되었습니다.");
        f.RequestUpdate();
      else
        QBar_userDefindedList[itemName]=nil;
        QBar_Print("|_1"..itemName.."|r|1은;는; 추가할 수 없는 아이템입니다."); 
      end
    end
  end
end
hooksecurefunc("ContainerFrameItemButton_OnModifiedClick",QBar_ContainerFrameItemButton_OnModifiedClick);
