local version = GetAddOnMetadata("simplemountequip","version");

local bonusNames = {
	spurs = "미스릴 박차",
	speed = "최하급 탈것 속도 증가",
	carrot = "당근 달린 지팡이",
	crop = "말채찍",
	charm = "날쌘 비행의 부적",
	whip = "하늘파괴자 채찍"
	--[[
	spurs = "Mithril Spurs",
	speed = "Minor Mount Speed Increase",
	carrot = "Carrot on a Stick",
	crop = "Riding Crop",
	whip = "Skybreaker Whip",
	charm = "Charm of Swift Flight",
	]]
};

local itemOrder = {
	{ "whip", "crop", "spurs", "speed", "carrot" },
	{ "charm" },
};

local backupItems = {
	spurs = "item:%p?%d+:464:", -- Mithril Spurs
	speed = "item:%p?%d+:930:", -- Minor Mount Speed Increase
	carrot = "item:11122:", -- Carrot on a stick
	crop = "item:25653:", -- Riding Crop
	whip = "item:32863:", -- Skybreaker Whip
	charm = "item:32481:", -- Charm of Swift Flight
};

local invSlots = { 8, 10, 13 };

local saveSlots = {
	spurs = 1,
	speed = 2,
	carrot = 3,
	crop = 3,
	whip = 3,
	charm = 3,
};

local mounted, combat, dead, controlled, wasCasting, doEquipCheck;
local tempGear, plrname;

local INCREASES = "^Increases speed by";
local INCREASESFLIGHT = "^Increases flight speed by";
if ( GetLocale() == "frFR" ) then
	INCREASES = "^Augmente la vitesse de";
elseif ( GetLocale() == "deDE" ) then
	INCREASES = "^Erh\195\182ht Tempo um";
	INCREASESFLIGHT = "^Fluggeschwindigkeit um";
elseif ( GetLocale() == "koKR" ) then
	INCREASES60 = "^이동 속도 60";
	INCREASES100 = "^이동 속도 100";
	INCREASESFLIGHT60 = "^비행 속도 60";
	INCREASESFLIGHT280 = "^비행 속도 280";
	INCREASESFLIGHT320 = "^비행 속도 320";
end

local frame = CreateFrame("Frame",nil,UIParent);
local tooltip = CreateFrame("GameTooltip", "smetip", frame, "GameTooltipTemplate");
local line1 = smetipTextLeft1;
local line2 = smetipTextLeft2;
tooltip:Show();
tooltip:SetOwner(frame,"ANCHOR_NONE");

local header = "|cffffffff자동박차 "..version.."|r : ";

local function print(msg)
	for text in string.gmatch(msg, "[^\n]+") do
		text = string.gsub(text,"%%1","|cffffcc00");
		text = string.gsub(text,"%%0(%S+)","|cff9999ff%1|r");
		DEFAULT_CHAT_FRAME:AddMessage(text,0.7,0.7,0.7);
	end
end

local function inBG()
	local inInstance, instanceType = IsInInstance()
	if inInstance and ( instanceType == "pvp" or instanceType == "arena" ) then
		return 1;
	end
end

local function delayedEquipCheck()
	local latency = select(3,GetNetStats());
	doEquipCheck = GetTime() + latency / 500;
end

local function wasCastingEvents()
	wasCasting = 1;
	frame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
	frame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	frame:RegisterEvent("UNIT_SPELLCAST_FAILED")
	frame:RegisterEvent("UNIT_SPELLCAST_STOP")
end

local function mountCheck()
	local i, name, rank, icon = 1, UnitAura("player",1,"HELPFUL|CANCELABLE");
	while name do
		if ( icon == "Interface\\Icons\\Ability_Druid_FlightForm" ) then
			return 2;
		end
		tooltip:SetOwner(frame,"ANCHOR_NONE");
		tooltip:SetUnitAura("player",name,rank);
		text = line2:GetText();
		--[[
		if ( text and ( string.match( text, INCREASES ) or string.match( text, INCREASESFLIGHT ) ) ) then
			return 1;
		end
		]]

		if ( text and string.match( text, INCREASES60 ) or text and string.match( text, INCREASES100 ) ) then
			return 1;
		elseif ( text and string.match( text, INCREASESFLIGHT60 ) or text and string.match( text, INCREASESFLIGHT280 ) or text and string.match( text, INCREASESFLIGHT320 ) ) then
			return 1;
		end

		i = i + 1;
		name, rank, icon = UnitAura("player",i,"HELPFUL|CANCELABLE");
	end
end

local function findItem( itemstring, simple )
	if ( itemstring ) then
		for i, slot in pairs(invSlots) do
			local item = GetInventoryItemLink("player", slot );
			if ( item and string.find( item, itemstring, simple, simple ) ) then
				return slot;
			end
		end
	end
	for bag = 0, 4 do
		local num = GetContainerNumSlots(bag);
		for slot = 1, num do
			local item = GetContainerItemLink(bag,slot);
			if ( item == itemstring or ( item and itemstring and string.find( item, itemstring, simple, simple ) ) ) then
				return bag, slot;
			end
		end
	end
end

local function gearCheck(t)
	for k in pairs(t) do
		return 1;
	end
end

local function equipItems()
	if ( combat or dead or controlled or UnitIsDeadOrGhost("player") ) then
		return;
	end
	if ( SpellIsTargeting() or UnitCastingInfo("player") or UnitChannelInfo("player") ) then
		wasCastingEvents();
		return;
	end
	local text = "";
	local o = SMountEquip_Saved;
	local gear = o[plrname].preferedGear;
	local quiet = o.quiet;
	local saved = {};
	local order = itemOrder[mounted];
	for i, bonus in pairs(order) do
		if ( not o[plrname][bonus] ) then
			local backupItem, bag, slot = backupItems[bonus];
			if ( gear and gear[bonus] ) then
				bag, slot = findItem( gear[bonus] );
			end
			if ( not slot ) then
				bag, slot = findItem( backupItem );
			end
			if ( slot ) then
				local saveSlot = saveSlots[bonus];
				if ( not IsInventoryItemLocked(invSlots[saveSlot]) ) then
					local saveItem = GetInventoryItemLink("player", invSlots[saveSlot]);
					if ( saveItem ) then
						saved[bonus] = string.match(saveItem,"(item:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+)");
					elseif ( SMountEquip_Saved.force ) then
						saved[bonus] = bag.."&"..slot;
					end
					if ( not quiet ) then
						local link = GetContainerItemLink(bag,slot);
						text = text.." "..link;
					end
					ClearCursor();
					PickupContainerItem(bag,slot);
					PickupInventoryItem(invSlots[saveSlot]);
					if ( saveSlot == 3 ) then
						break;
					end
				end
			end
			if ( bag and i == 1 ) then
				break;
			end
		end
	end
	if ( not quiet and #text > 0 ) then
		print( header.."착용"..text );
	end
	tempGear = nil;
	if ( not o[plrname].savedGear or not gearCheck(o[plrname].savedGear) ) then
		o[plrname].savedGear = saved;
	end
	ClearCursor();
end

local function unequipItems()
	if ( combat or dead or controlled or UnitIsDeadOrGhost("player") ) then
		return;
	end
	if ( SpellIsTargeting() or UnitCastingInfo("player") or UnitChannelInfo("player") ) then
		wasCastingEvents();
		return;
	end
	local text, total, recheck = "", 0;
	local quiet = SMountEquip_Saved.quiet;
	--[[if ( not tempGear and SMountEquip_Saved[plrname].savedGear ) then
		tempGear = SMountEquip_Saved[plrname].savedGear;
		SMountEquip_Saved[plrname].savedGear = nil;
	end--]]
	local tempGear = SMountEquip_Saved[plrname].savedGear
	if ( tempGear ) then
		for bonus, savedItem in pairs(tempGear) do
			total = total + 1;
			local bag, slot = strmatch( savedItem, "^(%d+)&(%d+)$" )
			if ( slot ) then
				if ( GetContainerItemLink(bag,slot) ) then
					bag, slot = findItem();
				end
			else
				bag, slot = findItem( savedItem, 1 );
			end
			if ( bag and slot ) then
				local invSlot, notInv = findItem(backupItems[bonus]);
				if ( invSlot and not notInv ) then
					recheck = 1;
					if ( not IsInventoryItemLocked(invSlot) ) then
						if ( not quiet ) then
							local link = GetContainerItemLink(bag,slot);
							text = text.." "..link;
						end
						ClearCursor();
						PickupInventoryItem(invSlot);
						PickupContainerItem(bag,slot);
					end
				end
			else
				--print( "Successfully equipped "..GetItemInfo(tempGear[bonus]) );
				tempGear[bonus] = nil;
				total = total - 1;
			end
		end
	end
	if ( recheck ) then
		frame:RegisterEvent("UNIT_INVENTORY_CHANGED");
	end
	if ( not quiet and #text > 0 ) then
		print( header.."착용"..text );
	end
	if ( total == 0 ) then
		SMountEquip_Saved[plrname].savedGear = nil;
	end
	ClearCursor();
end

local function equipCheck()
	local o = SMountEquip_Saved;
	oldMounted = mounted;
	mounted = mountCheck();
	if ( mounted ) then
		local enabled = ( not (not o.bg and inBG()) and not (o.nozone and o.nozone[GetRealZoneText()]) );
		if ( enabled ) then
			if ( not o.disable and ( not o[plrname].savedGear or ( oldMounted == 1 and mounted == 2 ) ) ) then
				equipItems();
			end
		elseif ( o[plrname].savedGear ) then
			unequipItems();
		end
	else
		mounted = nil;
		if ( o[plrname].savedGear ) then
			unequipItems();
		end
	end
end

local function trinketSlotSetup()
	if ( SMountEquip_Saved[plrname].trinket ) then
		invSlots[3] = 14;
	else
		invSlots[3] = 13;
	end
end

local function onEvent(self,event,arg1)
	if ( event == "PLAYER_REGEN_ENABLED" ) then
		combat = nil;
		equipCheck();
	elseif ( event == "PLAYER_REGEN_DISABLED" ) then
		combat = true;
	elseif ( event == "PLAYER_CONTROL_GAINED" ) then
		controlled = nil;
		equipCheck();
	elseif ( event == "PLAYER_CONTROL_LOST" ) then
		controlled = true
	elseif ( event == "PLAYER_DEAD" ) then
		dead = true;
	elseif ( event == "PLAYER_UNGHOST" ) then
		dead = nil;
		equipCheck();
	elseif ( event == "PLAYER_ALIVE" ) then
		dead = UnitIsDeadOrGhost("player");
		if ( not dead ) then
			equipCheck();
		end
	elseif ( event == "UNIT_INVENTORY_CHANGED" ) then
		if ( arg1 == "player" and not mounted ) then
			self:UnregisterEvent("UNIT_INVENTORY_CHANGED");
			unequipItems();
		end
	elseif ( event == "PLAYER_LOGOUT" ) then
		self:UnregisterAllEvents();
		self:SetScript("OnEvent", nil);
	elseif ( arg1 == "player" and not wasCasting or ( wasCasting and ( event == "UNIT_SPELLCAST_FAILED" or event == "UNIT_SPELLCAST_STOP" or event == "UNIT_SPELLCAST_SUCCEEDED" ) ) ) then
		delayedEquipCheck();
	end
end

frame:RegisterEvent("PLAYER_ENTERING_WORLD");
frame:SetScript("OnEvent",
	function(self,event)
		if ( event == "PLAYER_ENTERING_WORLD" ) then
			self:UnregisterEvent("PLAYER_ENTERING_WORLD");
			self:RegisterEvent("UNIT_AURA");

			plrname = UnitName("player") .. "-" .. GetRealmName();

			if ( not SMountEquip_Saved ) then
				SMountEquip_Saved = {};
			end

			if ( not SMountEquip_Saved[plrname] ) then
				SMountEquip_Saved[plrname] = {};
			end

			if ( SMountEquip_Saved.savedGear ) then
				if ( not SMountEquip_Saved[plrname].savedGear ) then
					SMountEquip_Saved[plrname].savedGear = SMountEquip_Saved.savedGear;
				end
				SMountEquip_Saved.savedGear = nil;
			end
			if ( SMountEquip_Saved.preferedGear ) then
				if ( not SMountEquip_Saved[plrname].preferedGear ) then
					SMountEquip_Saved[plrname].preferedGear = SMountEquip_Saved.preferedGear;
				end
				SMountEquip_Saved.preferedGear = nil;
			end

			trinketSlotSetup();

			self:SetScript("OnEvent",
				function(self)
					self:RegisterEvent("PLAYER_REGEN_ENABLED");
					self:RegisterEvent("PLAYER_REGEN_DISABLED");
					self:RegisterEvent("PLAYER_CONTROL_GAINED");
					self:RegisterEvent("PLAYER_CONTROL_LOST");
					self:RegisterEvent("PLAYER_DEAD");
					self:RegisterEvent("PLAYER_ALIVE");
					self:RegisterEvent("PLAYER_UNGHOST");

					self:RegisterEvent("UPDATE_SHAPESHIFT_FORMS");

					self:RegisterEvent("ZONE_CHANGED");
					self:RegisterEvent("ZONE_CHANGED_INDOORS");
					self:RegisterEvent("ZONE_CHANGED_NEW_AREA");

					self:RegisterEvent("PLAYER_LOGOUT");

					combat = UnitAffectingCombat("player");
					dead = UnitIsDeadOrGhost("player");
					equipCheck();

					self:SetScript("OnEvent", onEvent);
				end
				);
		end
	end
	);

frame:SetScript("OnUpdate",
	function()
		if ( doEquipCheck and doEquipCheck < GetTime() ) then
			doEquipCheck = nil;
			equipCheck();
		end
	end
	);

local function statusText(var, flip)
	if ( (var and not flip) or (not var and flip) ) then
		return "|cff00ff00활성화|r";
	end
	return "|cffff0000비활성화|r";
end

local function bgStatus()
	return " "..statusText(SMountEquip_Saved.bg).." 전장";
end

local function zoneStatus()
	local zones = SMountEquip_Saved.nozone;
	if ( zones ) then
		local hasZones;
		for zone in pairs(zones) do
			print("  "..statusText().." 지역: "..zone);
			hasZones = 1;
		end
		if ( not hasZones ) then
			zones = nil;
		end
	end
	print("  "..statusText(1).." 그 외 다른 지역");
end

local function trinketStatus()
	if ( SMountEquip_Saved[plrname].trinket ) then
		return "|cff9999ff모든 장신구|r를 두번째 장신구 슬롯에 착용";
	else
		return "|cff9999ff모든 장신구|r를 첫번째 장신구 슬롯에 착용";
	end
end

local function setZone(zone,status)
	if ( not SMountEquip_Saved.nozone ) then
		SMountEquip_Saved.nozone = {};
	end
	local zones = SMountEquip_Saved.nozone;
	if ( status == "disable" or ( status ~= "enable" and not zones[zone] ) ) then
		zones[zone] = true;
		print(header..statusText().." 지역: "..zone);
	else
		zones[zone] = nil;
		print(header..statusText(1).." 지역: "..zone);
	end
end

local function command(msg)
	msg = msg or "";
	local cmd = string.match(strlower(msg), "^(%a+)");
	if ( cmd == "disable" or cmd == "off" ) then
		SMountEquip_Saved.disable = true;
		print(header..statusText());
	elseif ( cmd == "enable" or cmd == "on" ) then
		SMountEquip_Saved.disable = nil;
		print(header..statusText(1));
	elseif ( cmd == "bg" or cmd == "battlegrounds" ) then
		local status = string.match(strlower(msg), "^"..cmd.." (%a+)");
		if ( status == "enable" or status == "on" ) then
			SMountEquip_Saved.bg = true;
			print(header..bgStatus());
		elseif ( status == "disable" or status == "off" ) then
			SMountEquip_Saved.bg = nil;
			print(header..bgStatus());
		elseif ( status == "toggle" ) then
			if ( SMountEquip_Saved.bg ) then
				SMountEquip_Saved.bg = nil;
			else
				SMountEquip_Saved.bg = true;
			end
			print(header..bgStatus());
		else
			print(header.."전장 옵션 -\n%1전장에 있는 동안에 장신구 교체를 활성화 혹은 비활성화합니다\n  %0on %1- 전장에서 활성화합니다\n  %0off %1- 전장에서 비활성화합니다\n  %0toggle %1- 전장에서의 설정을 전환합니다");
		end
	elseif ( cmd == "zone" ) then
		local status = string.match(strlower(msg), "^"..cmd.." (%a+)");
		local zone = GetRealZoneText();
		if ( status == "enable" or status == "on" ) then
			setZone(zone,"enable");
		elseif ( status == "disable" or status == "off" ) then
			setZone(zone,"disable");
		elseif ( status == "toggle" ) then
			setZone(zone);
		elseif ( status == "clear" ) then
			SMountEquip_Saved.nozone = nil;
			print(header.."비활성화된 지역 목록이 %0삭제되었습니다");
		else
			print(header.."지역 옵션 -\n%1현재 지역(|cff9999ff"..zone.."%1)에서 활성화 혹은 비활성화합니다\n  %0on %1- "..zone.."에서 활성화합니다\n  %0off %1- "..zone.."에서 비활성화합니다\n  %0toggle %1- "..zone.."에 대한 설정을 전환합니다\n  %0clear %1- 비활성화된 지역의 목록을 삭제합니다");
			--print(header.."zone options -\n%1enable or disable in the current zone (|cff9999ff"..zone.."%1)\n  %0on %1- enable in "..zone.."\n  %0off %1- disable in "..zone.."\n  %0toggle %1- toggle setting for "..zone.."\n  %0clear %1- clear list of disabled zones");
		end
	elseif ( cmd == "pref" or cmd == "prefered" ) then
		local item = string.match(msg, "(item:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+:%p?%d+)");
		if ( item ) then
			local name, link = GetItemInfo(item);
			for bonus, expression in pairs(backupItems) do
				if ( link and bonus ~= "carrot" and bonus ~= "crop" and string.match(item, expression) ) then
					if ( not SMountEquip_Saved[plrname].preferedGear ) then
						SMountEquip_Saved[plrname].preferedGear = {};
					end
					SMountEquip_Saved[plrname].preferedGear[bonus] = item;
					print(header..link.."|1이;가; 우선 순위의 |cff9999ff"..bonusNames[bonus].."|r 아이템으로 설정되었습니다;");
					return;
				end
			end
			print(header..link.."|1은;는; 탈것 속도 강화 아이템으로 보여지지 않습니다.");
		elseif ( string.match(strlower(msg), "^pref.* clear$" ) ) then
			SMountEquip_Saved[plrname].preferedGear = nil;
			print(header.."우선순위 착용구 설정이 %0삭제되었습니다");
		else
			print(header.."우선 순위 옵션 -\n%1 |cff9999ff"..bonusNames["spurs"].."%1 or |cff9999ff"..bonusNames["speed"].."%1의 탈것 강화 착용구에 사용할 우선 순위의 아이템을 설정합니다.\n  %0[아이템] %1- 링크에 추가하려면 소지품안의 아이템에 shift-클릭하세요\n  %0clear %1- 우선 순위 아이템의 저장된 목록을 삭제합니다");
			--print(header.."prefered options -\n%1set prefered items to use for your |cff9999ff"..bonusNames["spurs"].."%1 or |cff9999ff"..bonusNames["speed"].."%1 enchanted mount gear.\n  %0[item] %1- shift click an item in your inventory to add the link\n  %0clear %1- clear saved list of prefered items");
		end
	elseif ( cmd == "quiet" ) then
		if ( SMountEquip_Saved.quiet ) then
			SMountEquip_Saved.quiet = nil;
		else
			SMountEquip_Saved.quiet = 1;
		end
		print(header.."착용구 교체 알림 기능이 "..statusText(SMountEquip_Saved.quiet,1));
	elseif ( cmd == "trinket" ) then
		if ( SMountEquip_Saved[plrname].trinket ) then
			SMountEquip_Saved[plrname].trinket = nil;
		else
			SMountEquip_Saved[plrname].trinket = 1;
		end
		trinketSlotSetup();
		print( header .. trinketStatus() );
	elseif ( cmd == "noequip" ) then
		local bonus = string.match(strlower(msg), "^"..cmd.." (%a+)");
		if ( bonus and bonusNames[bonus] ) then
			if ( SMountEquip_Saved[plrname][bonus] ) then
				SMountEquip_Saved[plrname][bonus] = nil;
			else
				SMountEquip_Saved[plrname][bonus] = 1;
			end
			print(header.."|cff9999ff"..bonusNames[bonus].."|r has been "..statusText(SMountEquip_Saved[plrname][bonus],1));
		elseif ( bonus == "clear" ) then
			for bonus in pairs(bonusNames) do
				SMountEquip_Saved[plrname][bonus] = nil;
			end
			print(header.."비활성화된 보너스 유형의 목록이 |cff9999ff삭제|r되었습니다");
		else
			print(header.."착용 옵션 -\n%1각 탈것의 속도 보너스 유형의 착용을 전환합니다");
			for bonusType, bonusName in pairs(bonusNames) do
				print("  %0"..bonusType.." %1- 전환: |r"..bonusName);
			end
			print("  %0clear %1- 비활성화된 보너스 유형의 목록을 삭제합니다");
		end
	elseif ( cmd == "force" ) then
		if ( SMountEquip_Saved.force ) then
			SMountEquip_Saved[plrname].force = nil;
		else
			SMountEquip_Saved[plrname].force = 1;
		end
		print(header.."탈 때의 착용구 제거 강제 기능이 "..statusText(SMountEquip_Saved.force));
	elseif ( cmd == "status" ) then
		print(header..statusText(SMountEquip_Saved.disable,1));
		if ( not SMountEquip_Saved.disable ) then
			if ( not SMountEquip_Saved.disable ) then
				print(" "..bgStatus());
				zoneStatus();
			end
			if ( SMountEquip_Saved.quiet ) then
				print("  착용구 교체 알림 기능이 "..statusText());
			end
			if ( SMountEquip_Saved.force ) then
				print("  탈 때의 착용구 제거 강제 기능이 "..statusText(1));
			end
			print( "  " .. trinketStatus() );
			for bonus, bonusName in pairs(bonusNames) do
				if ( SMountEquip_Saved[plrname][bonus] ) then
					print("  |cff9999ff"..bonusName.."|r 보너스 "..statusText());
				end
			end
			if ( SMountEquip_Saved[plrname].preferedGear ) then
				local text = "  prefer to equip item(s)";
				for bonus, item in pairs(SMountEquip_Saved[plrname].preferedGear) do
					local name, link = GetItemInfo(item);
					if ( link ) then
						text = text.." "..link;
					else
						text = text.." (unknown item "..item..")";
					end
				end
				print(text);
			end
		end
	else
		print(header.."|cFFFF0000명령어 /sme \n  %0on %1- 애드온을 활성화합니다\n  %0off %1- 애드온을 비활성화합니다\n  %0bg [옵션] %1- 전장에서 활성화 혹은 비활성화합니다\n  %0zone [옵션] %1- 현재 지역에서 활성화 혹은 비활성화합니다\n  %0pref [옵션] %1- 우선 순위 탈것 착용구를 설정합니다\n  %0noequip [옵션] %1- 아이템 보너스 유형의 착용을 전환합니다\n  %0force %1- 탈때의 착용구 제거 강제 기능을 전환합니다\n  이미 있는 착용구에 착용하게 되는 경우에는 기본 아이템에 의해서만 제거됩니다\n  %0trinket %1- 장신구 슬롯을 교체합니다\n  %0quiet %1- 착용구 교체 알림 기능을 전환합니다\n  %0status %1- 현재 설정 상태를 표시합니다");
		--print(header.."options\n  %0on %1- enable addon\n  %0off %1- disable addon\n  %0bg [option] %1- enable or disable in battleground\n  %0zone [option] %1- enable or disable in current zone\n  %0pref [option] %1- set prefered mount gear\n  %0noequip [option] %1- toggle equipping of item bonus types\n  %0force %1- toggle forced removal of riding gear\n  by default items are only removed if it equipped over existing gear\n  %0trinket %1- swap trinket slots\n  %0quiet %1- toggle equipment swap announcement\n  %0status %1- current settings");
	end
	equipCheck();
end

SlashCmdList["SIMPLEMOUNTEQUIP"] = command;
SLASH_SIMPLEMOUNTEQUIP1 = "/sme";
SLASH_SIMPLEMOUNTEQUIP2 = "/smount";
SLASH_SIMPLEMOUNTEQUIP3 = "/smountequip";
SLASH_SIMPLEMOUNTEQUIP4 = "/simplemount";
SLASH_SIMPLEMOUNTEQUIP5 = "/simplemountequip";
