--[[ default 
local inColor = { 0.2, 1, 0.2, 1 }
local outColor = { 1, 0.5, 0.3, 1 }
local meleeIn = {[10] = { 1, 0.8, 0, 1 }, warning = { 1, 0.2, 0, 1 }}

local _, class = UnitClass("player")
local caster = {WARLOCK = true, MAGE = true, PRIEST = true}

local textmin = "maxm 이내"
local textin = "minm-maxm 이내"
local textout = "maxm 이상"

local function normal(self, text)
	self.text:SetText((text or textin):gsub("min", self.minrange):gsub("max", self.maxrange))
end

local function MinRange(self)
	if self.targetType and self.targetType == "harm" and caster[class] and not UnitIsDead(self.unit) then
		self.text:SetTextColor(unpack(meleeIn[self.maxrange] or meleeIn.warning))
	else
		self.text:SetTextColor(unpack(inColor))
	end
	if self.minrange == 0 then
		normal(self, textmin)
	else
		normal(self)
	end
end
local function MaxRange(self)
	if not self.minrange then
		self.text:SetTextColor(unpack(outColor))
		normal(self, textout)
	else
		self.text:SetTextColor(unpack(inColor))
		normal(self)
	end
end
local function new(frame)
	frame:SetClampedToScreen(true)

	frame:SetPoint("center")
	frame:SetWidth(100)
	frame:SetHeight(15)

	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function(self) if IsShiftKeyDown() then self:StartMoving() end end)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)

	frame.text = frame:CreateFontString(nil, "OVERLAY")
	frame.text:SetFont("Fonts\\2002.TTF", frame.unit == "target" and 20 or 14)
	frame.text:SetTextColor(1, 0.7, 0)
	frame.text:SetShadowOffset(1, -1)
	frame.text:SetPoint("CENTER", 0, 1)
end

new(T_Range("target", MinRange, MaxRange))
new(T_Range("focus", MinRange, MaxRange))

--]]

------------------------------------------------------------------------------------------------------------------
--[[ for hunter
local inColor = {0.2, 0.5, 1}
local outColor = {1, 0.2, 0}
local meleeIn = {0.2, 0.7, 0}

local textmin = "근접"
local textin = "원거리"
local textout = "거리 벗어남"

local function normal(self, text, color)
	local r,g,b = unpack(color)
	self.text:SetText(text)
	self:SetBackdropColor(r,g,b, 1)
	self:SetBackdropBorderColor(r,g,b, 1)
end
local function MinRange(self)
	if self.minrange == 0 then
		normal(self, textmin, meleeIn)
	else
		normal(self, textin, inColor)
	end
end
local function MaxRange(self)
	if not self.minrange then
		normal(self, textout, outColor)
	else
		normal(self, textin, inColor)
	end
end

local function new(frame)
	frame:SetClampedToScreen(true)

	frame:SetPoint("center")
	frame:SetWidth(100)
	frame:SetHeight(25)

	frame:SetBackdrop({
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
		edgeSize = 12, insets= {left = 2, right = 2, top = 2, bottom = 2}})
	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function(self) if IsShiftKeyDown() then self:StartMoving() end end)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)

	frame.text = frame:CreateFontString(nil, "OVERLAY")
	frame.text:SetFont("Fonts\\2002b.TTF", 12)
	frame.text:SetPoint("CENTER", 0, 1)
end

local function unitUpdate(self)
	if self.targetType ~= "harm" then self:Hide() end
end

new(T_Range("target", MinRange, MaxRange, nil, unitUpdate))
new(T_Range("focus", MinRange, MaxRange, nil, unitUpdate))
--]]


------------------------------------------------------------------------------------------------------------------
--[[ bar type ]]
local inColor = {0.2, 0.5, 1}
local outColor = {1, 0.2, 0}
local meleeIn = {0.2, 0.7, 0}

local textmin = "maxm 이내"
local textin = "minm-maxm 이내"
local textout = "maxm 이상"

local function normal(self, value, text, color, maxvalue)
	local r,g,b = unpack(color)
	self.bar:SetMinMaxValues(0, maxvalue or max(self.targetType and T_Range.maxspellrange[self.targetType] or 0, 28))
	self.bar:SetValue(value or 28)
	self.bar.gra:SetGradient("vertical", r/3, g/3, b/3, r, g, b)
	self.text:SetText(text:gsub("min", self.minrange):gsub("max", self.maxrange))
	self:SetBackdropBorderColor(r,g,b, 1)
end
local function MinRange(self)
	if self.minrange == 0 then
		-- self.targetType and T_Range.maxspellrange[self.targetType]
		normal(self, self.maxrange, textmin, meleeIn, self.maxrange)
	else
		normal(self, self.minrange, textin, inColor)
	end
end
local function MaxRange(self)
	if not self.minrange then
		normal(self, self.maxrange, textout, outColor, self.maxrange)
	else
		normal(self, self.minrange, textin, inColor)
	end
end

local function new(frame)
	frame:SetClampedToScreen(true)

	frame:SetPoint("center")

	frame:SetBackdrop({
		edgeFile = "Interface/Tooltips/UI-Tooltip-Background",
		bgFile = "Interface/Tooltips/UI-Tooltip-Background",
		edgeSize = 1.5, insets= {left = -1.2, right = -1.2, top = -1.2, bottom = -1.2}})
	frame:SetBackdropColor(0,0,0, 0.8)

	frame:EnableMouse(true)
	frame:SetMovable(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function(self) if IsShiftKeyDown() then self:StartMoving() end end)
	frame:SetScript("OnDragStop", frame.StopMovingOrSizing)

	frame.bar = CreateFrame("statusbar", nil, frame)
	frame.bar:SetStatusBarTexture("")
	frame.bar:SetPoint("topleft", 2, -2)
	frame.bar:SetPoint("bottomright", -2, 2)

	frame.bar.gra = frame.bar:GetRegions()
	frame.bar.gra:SetTexture(1, 1, 1, 1)

	frame.text = frame.bar:CreateFontString(nil, "OVERLAY")
	frame.text:SetPoint("CENTER", 0, 1)

	if frame.unit == "target" then
		frame:SetWidth(100)
		frame:SetHeight(20)
		frame.text:SetFont("Fonts\\2002b.TTF", 12, "outline")
	else
		frame:SetWidth(80)
		frame:SetHeight(18)
		frame.text:SetFont("Fonts\\2002b.TTF", 10)
	end

	T_PointFrames = T_PointFrames or {}
	T_PointFrames[frame] = true
	frame.nosort = true
end

new(T_Range("target", MinRange, MaxRange))
new(T_Range("focus", MinRange, MaxRange))
--]]