local ORIG_UnitFrameHealthBar_OnValueChanged = UnitFrameHealthBar_OnValueChanged;
local ORIG_TargetFrame_UpdateAuraPositions = TargetFrame_UpdateAuraPositions;
local ORIG_TargetFrame_UpdateAuras = TargetFrame_UpdateAuras;
local ORIG_RaidOptionsFrame_UpdatePartyFrames = RaidOptionsFrame_UpdatePartyFrames;
local ORIG_Target_Spellbar_AdjustPosition = Target_Spellbar_AdjustPosition;

-- 직업 아이콘 시작--
function asFixUnitFrame_OnLoad(self)

	self:RegisterEvent("PLAYER_TARGET_CHANGED");
	self:RegisterEvent("PARTY_MEMBERS_CHANGED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
	self:RegisterEvent("UNIT_HEALTH");
	self:RegisterEvent("UNIT_MAXHEALTH");

	hooksecurefunc("UnitFrame_UpdateThreatIndicator", asFixUnitFrame_UpdateThreatIndicator); 
end

function asFixUnitFrame_UpdateIcon(frame, unit)
	local icon = getglobal(frame.."ClassIcon");
	local texture = getglobal(frame.."ClassIconTexture");

	if (UnitIsPlayer(unit)) then
		local localizedClass, englishClass = UnitClass(unit);
		texture:SetTexture("Interface\\AddOns\\asFixUnitFrame\\Icons\\"..englishClass);
		icon:Show();
	else
		icon:Hide();
	end
end

function asFixUnitFrame_UpdatePartyIcons()
	
	local icon;
	local texture;

	if (HIDE_PARTY_INTERFACE == "1" and GetNumRaidMembers() > 0) then
		for index = 1, 4 do
			icon =  getglobal("PartyMemberFrame"..index.."ClassIcon");
			icon:Hide();
		end
		return;
	end	

	local _, englishClass;
	for index = 1, 4 do
		icon =  getglobal("PartyMemberFrame"..index.."ClassIcon");
		texture = getglobal("PartyMemberFrame"..index.."ClassIconTexture");

		_, englishClass = UnitClass("party"..index);
		
		if (englishClass ~= nil) then
			texture:SetTexture("Interface\\AddOns\\asFixUnitFrame\\Icons\\"..englishClass);
			icon:Show();
		else
			icon:Hide();
		end
	end
end

function RaidOptionsFrame_UpdatePartyFrames()
	
	ORIG_RaidOptionsFrame_UpdatePartyFrames();
	asFixUnitFrame_UpdatePartyIcons();

end

function asFixUnitFrame_OnEvent(self, event)
	if (event == "PLAYER_TARGET_CHANGED") then
		asFixUnitFrame_UpdateIcon("TargetFrame", "target");
		asTarget_HealthUpdate("target");
	elseif (event == "PARTY_MEMBERS_CHANGED" or event == "PLAYER_ENTERING_WORLD") then
		asFixUnitFrame_UpdatePartyIcons();
	elseif (event == "UNIT_HEALTH" or event == "UNIT_MAXHEALTH") then
		if (arg1 == "target") then
			asTarget_HealthUpdate("target");
		end
	end
end

-- 생명력 색상 시작 ---
function UnitFrameHealthBar_OnValueChanged(self, value)
	TextStatusBar_OnValueChanged(self, value);
	HealthBar_OnValueChanged(self, value, 1);
end

function asTarget_HealthUpdate(unit)
	if UnitIsConnected(unit) then
		local percent = UnitHealth(unit) / UnitHealthMax(unit);
		local r, g, b;
		if(percent > 0.5) then
			r = (1.0 - percent) * 2;
			g = 1.0;
		else
			r = 1.0;
			g = percent * 2;
		end
		b = 0;
		asTargetHealthPercent:SetText(string.format("%d%%", floor(percent * 100)));
		asTargetHealthPercent:SetTextColor(r, g, b)
		asTargetHealthPercent:Show()
	else
		asTargetHealthPercent:Hide()
	end
end

-- 버프 디버프 변경 시작 --
local AURA_START_X = 5;
local AURA_START_Y = 32;
local AURA_OFFSET_Y = 3;
local LARGE_AURA_SIZE = 24;
local SMALL_AURA_SIZE = 17;
local AURA_ROW_WIDTH = 122;
local TOT_AURA_ROW_WIDTH = 101;
local NUM_TOT_AURA_ROWS = 2;

function TargetFrame_UpdateAuraPositions(auraName, numAuras, numOppositeAuras, largeAuraList, updateFunc, maxRowWidth, offsetX)
	-- a lot of this complexity is in place to allow the auras to wrap around the target of target frame if it's shown

	-- Position auras
	local size;
	local offsetY = AURA_OFFSET_Y;
	-- current width of a row, increases as auras are added and resets when a new aura's width exceeds the max row width
	local rowWidth = 0;
	local firstBuffOnRow = 1;
	for i=1, numAuras do
		-- update size and offset info based on large aura status
		if ( largeAuraList[i] ) then
			size = LARGE_AURA_SIZE;
			offsetY = AURA_OFFSET_Y + AURA_OFFSET_Y;
		else
			size = SMALL_AURA_SIZE;
		end

		-- anchor the current aura
		if ( i == 1 ) then
			rowWidth = size;
			TargetFrame.auraRows = TargetFrame.auraRows + 1;
		else
			rowWidth = rowWidth + size + offsetX;
		end
		if ( rowWidth > maxRowWidth ) then
			-- this aura would cause the current row to exceed the max row width, so make this aura
			-- the start of a new row instead
			updateFunc(auraName, i, numOppositeAuras, firstBuffOnRow, size, offsetX, offsetY);

			rowWidth = size;
			TargetFrame.auraRows = TargetFrame.auraRows + 1;
			firstBuffOnRow = i;
			offsetY = AURA_OFFSET_Y;

			if ( TargetFrame.auraRows > NUM_TOT_AURA_ROWS ) then
				-- if we exceed the number of tot rows, then reset the max row width
				-- note: don't have to check if we have tot because AURA_ROW_WIDTH is the default anyway
				maxRowWidth = AURA_ROW_WIDTH;
			end
		else
			updateFunc(auraName, i, numOppositeAuras, i - 1, size, offsetX, offsetY);
		end
	end
end

local largeBuffList = {};
local largeDebuffList = {};

function TargetFrame_UpdateAuras (self)
	local button, buttonName;
	local buttonIcon, buttonCount, buttonCooldown, buttonStealable, buttonBorder;

	local name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable;
	local playerIsTarget = UnitIsUnit("player", "target");

	local numBuffs = 0;
	for i=1, MAX_TARGET_BUFFS do
		name, rank, icon, count, debuffType, duration, expirationTime, isMine, isStealable = UnitBuff("target", i);
		buttonName = "TargetFrameBuff"..i;
		button = _G[buttonName];
		if ( not button ) then
			if ( not icon ) then
				break;
			else
				button = CreateFrame("Button", buttonName, TargetFrame, "asTargetBuffButtonTemplate");
				button.unit = "target";
			end
		end
		if ( icon ) then
			button:SetID(i);

			-- set the icon
			buttonIcon = _G[buttonName.."Icon"];
			buttonIcon:SetTexture(icon);

			-- set the count
			buttonCount = _G[buttonName.."Count"];
			if ( count > 1 ) then
				buttonCount:SetText(count);
				buttonCount:Show();
			else
				buttonCount:Hide();
			end

			-- Handle cooldowns
			buttonCooldown = _G[buttonName.."Cooldown"];
			if ( duration > 0 ) then
				buttonCooldown:Show();
				CooldownFrame_SetTimer(buttonCooldown, expirationTime - duration, duration, 1);
			else
				buttonCooldown:Hide();
			end

			-- Show stealable frame if the target is not a player, the buff is stealable.
			buttonStealable = _G[buttonName.."Stealable"];
			if ( not playerIsTarget and isStealable ) then
				buttonStealable:Show();
			else
				buttonStealable:Hide();
			end
		
			buttonBorder = _G[buttonName.."Border"];
			buttonBorder:SetVertexColor(1.00, 1.00, 0.00);

			-- Set the buff to be big if the buff is cast by the player and the target is not the player
			largeBuffList[i] = (isMine and not playerIsTarget and not UnitIsEnemy("target", "player"));

			if  (isMine and not playerIsTarget and not UnitIsEnemy("target", "player")) then
				buttonBorder:SetWidth(LARGE_AURA_SIZE+2);
				buttonBorder:SetHeight(LARGE_AURA_SIZE+2);
			else
				buttonBorder:SetWidth(SMALL_AURA_SIZE+2);
				buttonBorder:SetHeight(SMALL_AURA_SIZE+2);
			end

			numBuffs = numBuffs + 1;

			button:ClearAllPoints();
			button:Show();
		else
			button:Hide();
		end
	end

	local color;
	local numDebuffs = 0;
	for i=1, MAX_TARGET_DEBUFFS do
		name, rank, icon, count, debuffType, duration, expirationTime, isMine = UnitDebuff("target", i);
		buttonName = "TargetFrameDebuff"..i;
		button = _G[buttonName];
		if ( not button ) then
			if ( not icon ) then
				break;
			else
				button = CreateFrame("Button", buttonName, TargetFrame, "TargetDebuffButtonTemplate");
				button.unit = "target";
			end
		end
		if ( icon ) then
			button:SetID(i);

			-- set the icon
			buttonIcon = _G[buttonName.."Icon"];
			buttonIcon:SetTexture(icon);

			-- set the count
			buttonCount = _G[buttonName.."Count"];
			if ( count > 1 ) then
				buttonCount:SetText(count);
				buttonCount:Show();
			else
				buttonCount:Hide();
			end

			-- Handle cooldowns
			buttonCooldown = _G[buttonName.."Cooldown"];
			if ( duration > 0 ) then
				buttonCooldown:Show();
				CooldownFrame_SetTimer(buttonCooldown, expirationTime - duration, duration, 1);
			else
				buttonCooldown:Hide();
			end

			-- set debuff type color
			if ( debuffType ) then
				color = DebuffTypeColor[debuffType];
			else
				color = DebuffTypeColor["none"];
			end
			buttonBorder = _G[buttonName.."Border"];
			buttonBorder:SetVertexColor(color.r, color.g, color.b);

			-- Set the buff to be big if the buff is cast by the player
			largeDebuffList[i] = isMine;

			numDebuffs = numDebuffs + 1;

			button:ClearAllPoints();
			button:Show();
		else
			button:Hide();
		end
	end

	TargetFrame.auraRows = 0;
	local haveTargetofTarget = TargetofTargetFrame:IsShown();
	local maxRowWidth;
	-- update buff positions
	maxRowWidth = ( haveTargetofTarget and TOT_AURA_ROW_WIDTH ) or AURA_ROW_WIDTH;
	TargetFrame_UpdateAuraPositions("TargetFrameBuff", numBuffs, numDebuffs, largeBuffList, TargetFrame_UpdateBuffAnchor, maxRowWidth, 3);
	-- update debuff positions
	maxRowWidth = ( haveTargetofTarget and TargetFrame.auraRows < NUM_TOT_AURA_ROWS and TOT_AURA_ROW_WIDTH ) or AURA_ROW_WIDTH;
	TargetFrame_UpdateAuraPositions("TargetFrameDebuff", numDebuffs, numBuffs, largeDebuffList, TargetFrame_UpdateDebuffAnchor, maxRowWidth, 4);
	-- update the spell bar position
	Target_Spellbar_AdjustPosition();
end

function Target_Spellbar_AdjustPosition ()
	local yPos = 5;
	if ( TargetFrame.auraRows ) then
		
		if ( TargetFrame.auraRows <= NUM_TOT_AURA_ROWS ) then
			--yPos = 38;
			yPos = LARGE_AURA_SIZE + SMALL_AURA_SIZE + 4
		else
			--yPos = 19 * TargetFrame.auraRows;
			yPos = LARGE_AURA_SIZE +  SMALL_AURA_SIZE * (TargetFrame.auraRows - 1) + 4;
		end
	end
	if ( TargetofTargetFrame:IsShown() ) then
		if ( yPos <= 25 ) then
			yPos = yPos + 25;
		end
	else
		yPos = yPos - 5;
		local classification = UnitClassification("target");
		if ( (yPos < 17) and ((classification == "worldboss") or (classification == "rareelite") or (classification == "elite") or (classification == "rare")) ) then
			yPos = 17;
		end
	end
	TargetFrameSpellBar:SetPoint("BOTTOM", "TargetFrame", "BOTTOM", -15, -yPos);
end

function asFixUnitFrame_UpdateThreatIndicator(indicator, numericIndicator, unit)
	if ( not indicator ) then
		return;
	end

	if ( not unit or unit == indicator.feedbackUnit ) then
		local status;
		if ( indicator.feedbackUnit ~= indicator.unit ) then
			status = UnitThreatSituation(indicator.feedbackUnit, indicator.unit);
		else
			status = UnitThreatSituation(indicator.feedbackUnit);
		end

		if ( status > 0 and IsThreatWarningEnabled() ) then
			indicator:SetVertexColor(GetThreatStatusColor(status));
			indicator:Show();

			if ( numericIndicator ) then
				if ( ShowNumericThreat() ) then
					local isTanking, status, percentage = UnitDetailedThreatSituation(indicator.feedbackUnit, indicator.unit);
					numericIndicator.text:SetText(format("%d", percentage).."%");
					numericIndicator.bg:SetVertexColor(GetThreatStatusColor(status));
					numericIndicator:Show();
				else
					numericIndicator:Hide();
				end
			end
		else
			indicator:Hide();
					
			if ( numericIndicator ) then
				if ( ShowNumericThreat()) then
					local isTanking, status, percentage = UnitDetailedThreatSituation(indicator.feedbackUnit, indicator.unit);
					if (percentage == nil or percentage == 0) then
						numericIndicator:Hide();
					else
						numericIndicator.text:SetText(format("%d", percentage).."%");
						numericIndicator.bg:SetVertexColor(GetThreatStatusColor(status));
						numericIndicator:Show();
					end
				else
					numericIndicator:Hide();
				end
			end
		end
	end
end
