﻿--[[-------------------------------------------------------------------------
  Copyright (c) 2008, Adrian L Lange
  Copyright (c) 2007-2008, Trond A Ekseth
  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are
  met:

      * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
      * Redistributions in binary form must reproduce the above
        copyright notice, this list of conditions and the following
        disclaimer in the documentation and/or other materials provided
        with the distribution.
      * Neither the name of pChat, oChat nor the names of its contributors may
        be used to endorse or promote products derived from this
        software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
---------------------------------------------------------------------------]]

-- settings
local c = { r = 1, g = .58, b = .09 } -- set your custom color here

-- the works
local dummy = function() end
local tex = "Interface\\AddOns\\pCast\\statusbar"

-- force targetcastbar to show
if GetCVar("ShowTargetCastbar")=="0" then
	SetCVar("ShowTargetCastbar", "1")
end

-- castbar
local cb = CastingBarFrame

local SetStatusBarColor = getmetatable(cb).__index.SetStatusBarColor
local _ssbc = function(self, ...)
	SetStatusBarColor(self, c.r, c.g, c.b)
end

cb.UnregisterEvent = dummy
cb.UnregisterAllEvents = dummy

cb:RegisterEvent"PLAYER_ENTERING_WORLD"
cb:RegisterEvent"UNIT_SPELLCAST_START"
cb:RegisterEvent"UNIT_SPELLCAST_STOP"
cb:RegisterEvent"UNIT_SPELLCAST_FAILED"
cb:RegisterEvent"UNIT_SPELLCAST_INTERRUPTED"
cb:RegisterEvent"UNIT_SPELLCAST_DELAYED"
cb:RegisterEvent"UNIT_SPELLCAST_CHANNEL_START"
cb:RegisterEvent"UNIT_SPELLCAST_CHANNEL_UPDATE"
cb:RegisterEvent"UNIT_SPELLCAST_CHANNEL_STOP"

cb:ClearAllPoints()
cb:SetPoint("BOTTOMRIGHT", PlayerFrame, "TOPRIGHT", -5, 2)
cb.SetPoint = dummy

cb:SetWidth(160)
cb:SetHeight(12)
cb:SetStatusBarTexture(tex)
cb.SetStatusBarColor = _ssbc

local bg, spell, border, icon, spark, flash = cb:GetRegions()
bg:ClearAllPoints()
bg:SetPoint("TOP", 0, 1.15)
bg:SetHeight(14)
bg:SetWidth(162)
bg:SetTexture(0,0,0,1)
bg.SetWidth = dummy
bg.SetHeight = dummy

spell:ClearAllPoints()
spell:SetPoint"CENTER"

spark.Show = dummy
flash.Show = dummy
spark:Hide()
icon:Hide()
flash:Hide()
border:Hide()

--[[
-- Castbar Timer
local text = cb:CreateFontString(nil, "ARTWORK")
text:SetFontObject(GameFontNormalSmall)
text:SetTextColor(1,1,1)
text:SetJustifyH"RIGHT"
text:SetPoint"RIGHT"
text:SetParent(cb)

local orig_CastingBarFrame_OnUpdate = CastingBarFrame_OnUpdate
function CastingBarFrame_OnUpdate(...)
	orig_CastingBarFrame_OnUpdate(...)
	if(CastingBarFrame.unit == "player") then
		if(this.casting and CastingBarFrame.maxValue) then
			text:SetText(format("%.1f", CastingBarFrame.maxValue - GetTime()))
			text:Show()
		elseif(this.channeling and CastingBarFrame.maxValue) then
			local _, _, _, _, _, endTime = UnitCastingInfo("player")
			text:SetText(format("%.1f", CastingBarFrame.endTime - GetTime()))
			text:Show()
		else
			text:Hide()
		end
	end
end
--]]

-- Target Castbar
local tb = TargetFrameSpellBar

tb.UnregisterEvent = dummy
tb.UnregisterAllEvents = dummy

tb:RegisterEvent"CVAR_UPDATE"
tb:RegisterEvent"PLAYER_TARGET_CHANGED"
tb:RegisterEvent"UNIT_SPELLCAST_START"
tb:RegisterEvent"UNIT_SPELLCAST_STOP"
tb:RegisterEvent"UNIT_SPELLCAST_FAILED"
tb:RegisterEvent"UNIT_SPELLCAST_INTERRUPTED"
tb:RegisterEvent"UNIT_SPELLCAST_DELAYED"
tb:RegisterEvent"UNIT_SPELLCAST_CHANNEL_START"
tb:RegisterEvent"UNIT_SPELLCAST_CHANNEL_UPDATE"
tb:RegisterEvent"UNIT_SPELLCAST_CHANNEL_STOP"

--tb:SetParent(UIParent)
tb:ClearAllPoints()
--tb:SetPoint("CENTER", UIParent, 0, -300)
tb:SetPoint("BOTTOMLEFT", TargetFrame, "TOPLEFT", 5, 2)
tb.SetPoint = dummy ------

tb:SetWidth(176)
tb:SetHeight(13.5)
tb:SetStatusBarTexture(tex)
tb.SetStatusBarColor = _ssbc

local bg, spell, border, icon, spark, flash = tb:GetRegions()
bg:ClearAllPoints()
bg:SetPoint("TOP", 0, 1.15)
bg:SetHeight(15.5)
bg:SetWidth(178)
bg:SetTexture(0,0,0,1)
bg.SetWidth = dummy
bg.SetHeight = dummy

spell:ClearAllPoints()
spell:SetPoint"CENTER"

spark.Show = dummy
flash.Show = dummy
flash:Hide()
icon:Hide()
spark:Hide()
border:Hide()

Target_Spellbar_AdjustPosition = dummy